Package: adoptr
Type: Package
Title: Adaptive Optimal Two-Stage Designs
Version: 1.1.2
Authors@R: c(person("Kevin", "Kunzmann", role = c("aut", "cph"), email = "kevin.kunzmann@boehringer-ingelheim.com",comment = c(ORCID = "0000-0002-1140-7143")),
        person("Maximilian", "Pilz", role = c("aut", "cre"), email = "maximilian.pilz@th-nuernberg.de", comment = c(ORCID = "0000-0002-9685-1613")),
        person("Jan", "Meis", role = c("aut"), email = "meis@imbi.uni-heidelberg.de", comment = c(ORCID = "0000-0001-5407-7220")),
        person("Nico", "Bruder", role = c("aut"), email = "bruder@imbi.uni-heidelberg.de", comment = c(ORCID = "0009-0004-9522-2075")))
Description: Optimize one or two-arm, two-stage designs for clinical trials with 
    respect to several implemented objective criteria or custom objectives.
    Optimization under uncertainty and conditional (given stage-one outcome) 
    constraints are supported.
    See Pilz et al. (2019) <doi:10.1002/sim.8291> and 
    Kunzmann et al. (2021) <doi:10.18637/jss.v098.i09> for details.
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat, covr, rpact, vdiffr, pwr, dplyr,
        ggplot2, tidyr, gridExtra, bookdown
Imports: nloptr, methods, glue
VignetteBuilder: knitr
Collate: 'DataDistribution.R' 'BinomialDistribution.R'
        'ChiSquaredDistribution.R' 'Prior.R' 'TwoStageDesign.R'
        'OneStageDesign.R' 'util.R' 'Scores.R' 'CompositeScore.R'
        'ConditionalPower.R' 'ConditionalSampleSize.R'
        'ContinuousPrior.R' 'FDistribution.R' 'GroupSequentialDesign.R'
        'MaximumSampleSize.R' 'NormalDistribution.R' 'PointMassPrior.R'
        'StudentDistribution.R' 'Survival.R' 'adoptr.R' 'constraints.R'
        'minimize.R' 'regularization.R'
BugReports: https://github.com/optad/adoptr/issues
URL: https://github.com/optad/adoptr, https://optad.github.io/adoptr/
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-02 20:34:55 UTC; maximilian
Author: Kevin Kunzmann [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-1140-7143>),
  Maximilian Pilz [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9685-1613>),
  Jan Meis [aut] (ORCID: <https://orcid.org/0000-0001-5407-7220>),
  Nico Bruder [aut] (ORCID: <https://orcid.org/0009-0004-9522-2075>)
Maintainer: Maximilian Pilz <maximilian.pilz@th-nuernberg.de>
Repository: CRAN
Date/Publication: 2026-05-03 13:10:02 UTC
Built: R 4.6.0; ; 2026-05-03 14:24:15 UTC; unix
