# Generated by roxygen2: do not edit by hand

S3method(print, TwoStageDesignSummary)
export(ANOVA)
export(AverageN2)
export(Binomial)
export(ChiSquared)
export(ConditionalPower)
export(ConditionalSampleSize)
export(ContinuousPrior)
export(ExpectedNumberOfEvents)
export(ExpectedSampleSize)
export(GroupSequentialDesign)
export(MaximumSampleSize)
export(N1)
export(NestedModels)
export(Normal)
export(OneStageDesign)
export(Pearson2xK)
export(PointMassPrior)
export(Power)
export(Student)
export(Survival)
export(SurvivalDesign)
export(TwoStageDesign)
export(ZSquared)
export(bounds)
export(c2)
export(composite)
export(condition)
export(cumulative_distribution_function)
export(evaluate)
export(expectation)
export(expected)
export(get_initial_design)
export(get_lower_boundary_design)
export(get_tau_ANOVA)
export(get_tau_Pearson2xK)
export(get_tau_ZSquared)
export(get_upper_boundary_design)
export(make_fixed)
export(make_tunable)
export(minimize)
export(n)
export(n1)
export(n2)
export(posterior)
export(predictive_cdf)
export(predictive_pdf)
export(probability_density_function)
export(subject_to)
export(tunable_parameters)
exportClasses(ANOVA)
exportClasses(AverageN2)
exportClasses(Binomial)
exportClasses(ChiSquared)
exportClasses(ConditionalPower)
exportClasses(ConditionalSampleSize)
exportClasses(ContinuousPrior)
exportClasses(DataDistribution)
exportClasses(GroupSequentialDesign)
exportClasses(GroupSequentialDesignSurvival)
exportClasses(MaximumSampleSize)
exportClasses(N1)
exportClasses(NestedModels)
exportClasses(Normal)
exportClasses(OneStageDesign)
exportClasses(OneStageDesignSurvival)
exportClasses(Pearson2xK)
exportClasses(PointMassPrior)
exportClasses(Prior)
exportClasses(Student)
exportClasses(Survival)
exportClasses(TwoStageDesign)
exportClasses(TwoStageDesignSurvival)
exportClasses(ZSquared)
exportMethods("<=")
exportMethods(">=")
exportMethods(GroupSequentialDesign)
exportMethods(OneStageDesign)
exportMethods(SurvivalDesign)
exportMethods(TwoStageDesign)
exportMethods(bounds)
exportMethods(c2)
exportMethods(condition)
exportMethods(cumulative_distribution_function)
exportMethods(evaluate)
exportMethods(expectation)
exportMethods(expected)
exportMethods(get_lower_boundary_design)
exportMethods(get_upper_boundary_design)
exportMethods(make_fixed)
exportMethods(make_tunable)
exportMethods(n)
exportMethods(n1)
exportMethods(n2)
exportMethods(plot)
exportMethods(posterior)
exportMethods(predictive_cdf)
exportMethods(predictive_pdf)
exportMethods(probability_density_function)
exportMethods(quantile)
exportMethods(simulate)
exportMethods(summary)
exportMethods(tunable_parameters)
exportMethods(update)
import(methods)
